package hw.tagApi.service.utils;

import hw.tagApi.service.domain.ApiContent;
import hw.tagApi.service.domain.ApiResponse;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.fasterxml.jackson.databind.JsonNode;

import java.io.IOException;
import java.util.List;

import static hw.tagApi.service.constant.ApiConstants.WECHAT_WEBHOOK_URL;

public class httpClientUtils {
    private static final Logger log = LoggerFactory.getLogger(httpClientUtils.class);
    private static final ObjectMapper objectMapper = new ObjectMapper();
    private static final CloseableHttpClient httpClient = HttpClients.createDefault();

    // 超时配置
    private static final int CONNECT_TIMEOUT = 5000;
    private static final int SOCKET_TIMEOUT = 10000;
    private static final RequestConfig requestConfig = RequestConfig.custom()
            .setConnectTimeout(CONNECT_TIMEOUT)
            .setSocketTimeout(SOCKET_TIMEOUT)
            .build();

    // 重试配置
    private static final int MAX_RETRIES = 2;
    private static final int RETRY_INTERVAL = 1000; // 1秒

    /**
     * 创建并配置HttpGet请求
     *
     * @param url 请求URL
     * @return 配置好的HttpGet请求对象
     */
    public static HttpGet createHttpGetRequest(String url) {
        HttpGet httpGet = new HttpGet(url);
        // 设置请求头
        httpGet.setHeader("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36");
        httpGet.setHeader("Accept", "*/*");
        httpGet.setHeader("Accept-Language", "zh-CN,zh;q=0.9,en;q=0.8");
        httpGet.setHeader("Connection", "keep-alive");
        return httpGet;
    }

    /**
     * 从URL获取字节数组
     *
     * @param url 请求URL
     * @return 字节数组，如果发生错误则返回null
     */
    public static byte[] getByteArrayFromUrl(String url) {
        HttpGet httpGet = createHttpGetRequest(url);
        try (CloseableHttpResponse response = httpClient.execute(httpGet)) {
            // 获取响应状态码
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != 200) {
                log.error("获取文件失败，响应码: {}", statusCode);
                return null;
            }

            // 获取响应实体
            HttpEntity entity = response.getEntity();
            if (entity == null) {
                log.error("无法获取响应实体");
                return null;
            }

            // 将响应实体转换为字节数组
            byte[] fileData = EntityUtils.toByteArray(entity);

            if (fileData.length == 0) {
                log.error("获取到的文件为空");
                return null;
            }

            log.info("成功获取文件，大小: {} 字节", fileData.length);
            return fileData;
        } catch (IOException e) {
            log.error("获取文件时发生错误: {}", e.getMessage(), e);
            return null;
        }
    }

    /**
     * 发送处理结果到回调接口，支持重试机制
     *
     * @param resList  处理结果列表
     * @param url      回调URL
     * @param caseCode 案例代码
     */
    public static void sendResultToCallback(List<ApiContent> resList, String url, String caseCode) {
        if (resList == null || resList.isEmpty()) {
            log.warn("处理结果列表为空，不发送回调请求");
            return;
        }

        HttpPost httpPost = null;
        int retryCount = 0;
        boolean success = false;
        String errorMessage = "";
        String jsonBody = "";

        while (!success && retryCount < MAX_RETRIES) {
            try {
                // 创建POST请求
                httpPost = new HttpPost(url);
                httpPost.setConfig(requestConfig);

                // 设置请求头
                httpPost.setHeader("Content-Type", "application/json");
                httpPost.setHeader("Accept", "application/json");
                httpPost.setHeader("Connection", "close");

                // 创建响应对象
                ApiResponse response = new ApiResponse();
                response.setCaseCode(caseCode);
                response.setResult(resList);

                // 将对象转换为JSON字符串
                jsonBody = objectMapper.writeValueAsString(response);
//                log.info("准备发送的数据: {}", jsonBody);

                // 设置请求体
                StringEntity entity = new StringEntity(jsonBody, "UTF-8");
                httpPost.setEntity(entity);

                // 发送请求
                try (CloseableHttpResponse httpResponse = httpClient.execute(httpPost)) {
                    int statusCode = httpResponse.getStatusLine().getStatusCode();
                    HttpEntity responseEntity = httpResponse.getEntity();
                    String responseBody = responseEntity != null ? EntityUtils.toString(responseEntity, "UTF-8") : "";

                    if (statusCode == 200) {
//                        log.info("发送处理结果到回调接口成功，响应状态码: {}", statusCode);
                        log.info("回调接口响应内容: {}", responseBody);
                        success = true;
                    } else {
                        errorMessage = String.format("回调失败 - 状态码: %d, 响应内容: %s, 请求数据: %s",
                                statusCode, responseBody, jsonBody);
                        log.error(errorMessage);
                        retryCount++;
                        if (retryCount < MAX_RETRIES) {
                            log.info("将在{}秒后进行第{}次重试", RETRY_INTERVAL / 1000, retryCount + 1);
                            Thread.sleep(RETRY_INTERVAL);
                        }
                    }
                }
            } catch (Exception e) {
                errorMessage = String.format("回调失败 - 异常信息: %s, 请求数据: %s", e.getMessage(), jsonBody);
                log.error(errorMessage);
                retryCount++;
                if (retryCount < MAX_RETRIES) {
                    log.info("catch将在{}秒后进行第{}次重试", RETRY_INTERVAL / 1000, retryCount + 1);
                    try {
                        Thread.sleep(RETRY_INTERVAL);
                    } catch (InterruptedException ie) {
                        Thread.currentThread().interrupt();
                    }
                }
            } finally {
                if (httpPost != null) {
                    httpPost.releaseConnection();
                }
            }
        }

        if (!success) {
            String finalMessage = String.format("回调接口调用失败 - CASE代码: %s\n重试次数: %d\n错误信息: %s",
                    caseCode, MAX_RETRIES, errorMessage);
            log.info(finalMessage);
            // 发送企业微信通知
            sendWechatNotification(finalMessage);
        }
    }

    /**
     * 发送企业微信通知
     *
     * @param message 通知消息
     */
    public static void sendWechatNotification(String message) {
        try {
            String webhookUrl = WECHAT_WEBHOOK_URL;
            HttpPost httpPost = new HttpPost(webhookUrl);
            httpPost.setConfig(requestConfig);
            httpPost.setHeader("Content-Type", "application/json");

            // 构建消息体
            ObjectNode rootNode = objectMapper.createObjectNode();
            rootNode.put("msgtype", "text");
            ObjectNode textNode = rootNode.putObject("text");
            textNode.put("content", message);

            String jsonBody = objectMapper.writeValueAsString(rootNode);
            StringEntity entity = new StringEntity(jsonBody, "UTF-8");
            httpPost.setEntity(entity);

            try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
                HttpEntity responseEntity = response.getEntity();
                if (responseEntity != null) {
                    String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
                    JsonNode jsonResponse = objectMapper.readTree(responseBody);
                    int errcode = jsonResponse.get("errcode").asInt();
                    String errmsg = jsonResponse.get("errmsg").asText();
                    if (errcode == 0 && "ok".equals(errmsg)) {
                        log.info("企业微信通知发送成功");
                    } else {
                        log.error("企业微信通知发送失败 - errcode: {}, errmsg: {}", errcode, errmsg);
                    }
                }
            }
        } catch (Exception e) {
            log.error("发送企业微信通知时发生错误: {}", e.getMessage());
        }
    }

}
